@@ -359,7 +359,7 @@ maybe_append_separator_to_path (GtkFileChooserEntry *chooser_entry,
 				GtkFilePath         *path,
 				gchar               *display_name)
 {
-  if (path)
+  if (!g_str_has_suffix (display_name, G_DIR_SEPARATOR_S) && path)
     {
       GtkFileInfo *info;
 	    
@@ -377,7 +377,6 @@ maybe_append_separator_to_path (GtkFileChooserEntry *chooser_entry,
 	  
 	  gtk_file_info_free (info);
 	}
-
     }
 
   return display_name;
@@ -572,9 +571,11 @@ update_current_folder_files (GtkFileChooserEntry *chooser_entry,
 				       NULL); /* NULL-GError */
       if (info)
 	{
-	  const gchar *display_name = gtk_file_info_get_display_name (info);
+	  gchar *display_name = g_strdup (gtk_file_info_get_display_name (info));
 	  GtkTreeIter iter;
 
+          display_name = maybe_append_separator_to_path (chooser_entry, path, display_name);
+
 	  gtk_list_store_append (chooser_entry->completion_store, &iter);
 	  gtk_list_store_set (chooser_entry->completion_store, &iter,
 			      DISPLAY_NAME_COLUMN, display_name,
@@ -582,6 +583,7 @@ update_current_folder_files (GtkFileChooserEntry *chooser_entry,
 			      -1);
 
 	  gtk_file_info_free (info);
+          g_free (display_name);
 	}
     }
 
@@ -646,8 +648,6 @@ out:
 static gboolean
 load_directory_callback (GtkFileChooserEntry *chooser_entry)
 {
-  GSList *child_paths = NULL;
-
   GDK_THREADS_ENTER ();
 
   chooser_entry->load_directory_idle = 0;
@@ -848,8 +848,7 @@ gtk_file_chooser_entry_changed (GtkEditable *editable)
 	file_part_pos = 0;
     }
 
-  if (chooser_entry->file_part)
-    g_free (chooser_entry->file_part);
+  g_free (chooser_entry->file_part);
 
   chooser_entry->file_part = file_part;
   chooser_entry->file_part_pos = file_part_pos;
